function U=ab2u(A,B)

% U=ab2u(A,B) : solutions of S-A*S*A' = B*H + H'*B' are in kernel(U')
%
% Given A,B (complex-valued) state matrices of an IS-system
% 
% U = [u(1) u(2) ... ]
%
% is such that any solution S of S-A*S*A' = B*H + H'*B'
% is in the kernel of U' and conversely, this kernel characterizes all
% solutions. The mapping between matrices and vectors respects inner
% products and follows the convention in mat2vec and vec2mat.
%
% Alternatively, trace(vec2mat(u(i))*S)=0 for all i.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[n,m]=size(B);
[u,s,v]=svd(B); utB=u'*B;
                T=[inv(utB(1:m,1:m)) zeros(m,n-m); zeros(n-m,m) eye(n-m)]*u';
                invT=u*[utB(1:m,1:m) zeros(m,n-m); zeros(n-m,m) eye(n-m)];
a=T*A*invT;

range=[]; % range of H \mapsto S : S-ASA'=BH+H'B'

for k=1:m,
    Q=zeros(n,n); Q(k,k)=1;
    S=dlyap(a,Q);
    range=[range mat2vec(invT*S*invT')];
end

for k=2:m, for ell=1:k-1,
        Q=zeros(n,n); 
                     Q(k,ell)=1; Q(ell,k)=1;
                     S=dlyap(a,Q);
                     range=[range mat2vec(invT*S*invT')];
      
        Q=zeros(n,n); 
                     Q(k,ell)=i; Q(ell,k)=-i;
                     S=dlyap(a,Q);
                     range=[range mat2vec(invT*S*invT')];
end, end

for k=1:m, for ell=m+1:n,
        Q=zeros(n,n); 
                     Q(k,ell)=1; Q(ell,k)=1;
                     S=dlyap(a,Q);
                     range=[range mat2vec(invT*S*invT')];
        Q=zeros(n,n); 
                     Q(k,ell)=i; Q(ell,k)=-i;
                     S=dlyap(a,Q);
                     range=[range mat2vec(invT*S*invT')];
end, end

U=null(range');

% test
% hrand=rand(m,n); Prand=dlyap(A,B*hrand+hrand'*B');
% for k=1:n1-N, trace(vec2mat(U(k))*Prand), end
%     
% Stest=invT*S*invT';svd([Stest-A*Stest*A' B;B' zeros(m,m)]), pause