function v=mat2vec(M)

% v=mat2vec(M) : matrix M to vector v scaled to preserve the 2-norm
%
% M hermitian is stacked into a column vector
%   keeping only the top triangular part
%
% M11 M12 M13 ...
%     M22 M23 ...
% 
% into  [M11;
%        M22;
%        ...
%        Mnn;
%        real(M12) * sqrt(2);
%        real(M13) * sqrt(2);
%        ...
%        imag(M12) * sqrt(2);
%        ...]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n=length(M);
   Mreal=real(M);
   Mimag=imag(M);
   Mtopreal=diag(Mreal); Mtopimag=[];
   for i=2:n;
   Mtopreal=[Mtopreal; sqrt(2)*Mreal(1:i-1,i)];
   Mtopimag=[Mtopimag; sqrt(2)*Mimag(1:i-1,i)];
   end

v=[Mtopreal; Mtopimag];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% based on cvec.m but changed so as to preserve 2-norms