function M=Vec2matrix(v)

% M=vec2mat(v) : vector v is packaged into a Hermitian matrix M
%
% M hermitian formed from a column (complex) vector v
%   which contains the coefficients of the top triangular part of M as
%   follows:
%
% M = [M11 M12 M13 ...
%      M12 M22 M23 ...
% 
% from  [M11;
%        M22;
%        ...
%        Mnn;
%        real(M12) * sqrt(2);
%        real(M13) * sqrt(2);
%        ...
%        imag(M12) * sqrt(2);
%        ...]
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
v=v(:);
N=length(v);
% N = n+2*n*(n-1)/2 = n^2.

n=sqrt(N); n=round(n);
if N~=n^2, disp('the length of v ought to be a perfect square, and it is not'),
return, end

M=diag(v(1:n)); v(1:n)=[];
v=sqrt(2)*v; vr=v(1:n*(n-1)/2); v(1:n*(n-1)/2)=[]; v=vr-1i*v;

for i=2:n,
    M(1:i-1,i)=v(1:i-1);
    v(1:i-1)=[];
end

M=(M+M')/2;