function [traceQQh,Tval,Qval,Qhval]=MAapprox(Th,m)
% function [trace(Q+Qh), T, Q, Qh]=MA_approx(Th,m)
%
%         T + Q = Th + Qh
% where T is Toeplitz and corresponds to a MA(m) process
% Q, Qh >0 and trace(Q+Qh) is minimal.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n1=length(Th); Neps=1e-10; m1=m+1; n=n1-1;
kk=1:m1; 
if m>n,   tstructure=toeplitz([kk]);                      M=eye(n1,m1);
          f=eye(1,m1); g=eye(m1); g(:,1)=[];
else,
          zz=zeros(1,n-m); tstructure=toeplitz([kk zz]);  M=eye(n1);
          f=eye(1,n1); g=eye(n1); g=g(:,2:m+1);
end

setlmis([])
T=lmivar(3,tstructure);
Q=lmivar(1,[n1,1]);
Qh=lmivar(1,[n1,1]);
P=lmivar(1,[m,1]);
A=compan(eye(1,m1)); B=eye(m,1);
%
lmiterm([1 1 1 T],M,M')
lmiterm([-1 1 1 0],Th+eye(n1,n1)*Neps);
lmiterm([1 1 1 Q],1,1);
lmiterm([-1 1 1 Qh],1,1);
%
lmiterm([2 1 1 0],Th-eye(n1,n1)*Neps)
lmiterm([-2 1 1 T],M,M');
lmiterm([2 1 1 Qh],1,1);
lmiterm([-2 1 1 Q],1,1);
%
lmiterm([-3 1 1 Q],1,1);
lmiterm([3 1 1 0],0);
%
lmiterm([-4 1 1 Qh],1,1);
lmiterm([4 1 1 0],0);
%
%%%%% 0 < [p-A'*p*A g'*Tval*f'-A'*p*B; f*Tval*g-B'*p*A f*Tval*f'-B'*p*B]
lmiterm([-5 1 1 P],1,1);lmiterm([-5 1 1 P],-A',A);
lmiterm([-5 1 2 T],g',f');lmiterm([-5 1 2 P],-A',B);
%lmiterm([-5 2 1 T],f,g);lmiterm([-5 2 1 P],-B',A);
lmiterm([-5 2 2 T],f,f');lmiterm([-5 2 2 P],-B',B);
lmiterm([5 1 1 0],0);
%
lmiterm([-6 1 1 P],1,1);
lmiterm([6 1 1 0],0);
%
lmisys=getlmis;

weight=eye(n1,n1); zeroP=zeros(m,m); zeroT=zeros(size(tstructure));
c=mat2dec(lmisys,zeroT,weight,weight,zeroP);
options=[1e-12,0,0,0,1];
[copt,xopt]=mincx(lmisys,c,options);

Tval=dec2mat(lmisys,xopt,T);
Qval=dec2mat(lmisys,xopt,Q);
Qhval=dec2mat(lmisys,xopt,Qh); %p=dec2mat(lmisys,xopt,P);
traceQQh=trace(Qval+Qhval);
% Written by T.T. Georgiou (December 2006)