function [tresidual,tma]=MAdecomp(T,m)
% function [tresidual,tma]=MAdecomp(T,m)
%
%        T = Tresidual + Tma  (with Tma MA(m), Toeplitz, and of max
%                              trace subject to > 0 for all matrices)
%
% where Tma is a Toeplitz matrix corresponding to an MA(m) process
% with spectrum r0+2*r1*cos(theta)+...+2*rm*cos(m*theta),
% T,Tresidual are positive semidefinite and Tma has maximal trace
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% below tresidual are values, Tresidual ar variables, etc.
narg=nargin; if narg<2, disp('needs at least 2 arguments'), return, end

if m==0, tma=eye(size(T))*min(eig(T)); tresidual=T-tma; return, end
n1=length(T); Neps=1e-15; m1=m+1; n=n1-1; kk=1:m1; 
if m>n,  tstructure=toeplitz([kk]);                     M=eye(n1,m1);
         f=eye(1,m1); g=eye(m1); g(:,1)=[];
         %
else,    zz=zeros(1,n-m); tstructure=toeplitz([kk zz]); M=eye(n1);
         f=eye(1,n1); g=eye(n1); g=g(:,2:m+1);
end

setlmis([])
Tma=lmivar(3,tstructure);   P=lmivar(1,[m,1]);
A=compan(eye(1,m1));        B=eye(m,1);
%%%%% Tma < T
lmiterm([1 1 1 Tma],M,M')
lmiterm([-1 1 1 0],T);

%%%%% 0 < [p-A'*p*A g'*tma*f'-A'*p*B; f*tma*g-B'*p*A f*tma*f'-B'*p*B]
lmiterm([-2 1 1 P],1,1);lmiterm([-2 1 1 P],-A',A);
lmiterm([-2 1 2 Tma],g',f');lmiterm([-2 1 2 P],-A',B);
%lmiterm([-2 2 1 Tma],f,g);lmiterm([-2 2 1 P],-B',A);
lmiterm([-2 2 2 Tma],f,f');lmiterm([-2 2 2 P],-B',B);
lmiterm([2 1 1 0],0);

%%%%% 0 < P
lmiterm([-3 1 1 P],1,1);
lmiterm([3 1 1 0],0);

lmisys=getlmis;
if m>n,     weights=eye(m+1,m+1);
else,       weights=eye(n+1,n+1);  end

weight0=zeros(m,m);
c=mat2dec(lmisys,weights,weight0);
options=[1e-15,0,0,0,1];
[copt,xopt]=mincx(lmisys,-c,options);
trcq=-copt;

tma=dec2mat(lmisys,xopt,Tma);  tresidual=T-M*tma*M';
p=dec2mat(lmisys,xopt,P);
% Written by T.T. Georgiou (August 2005, revised January 2007)